#include "StdAfx.h"
#include "MyListCtrl.h"
#include "globals.h"

using namespace std;

int CMyListCtrl::m_nRecursionCount = 0;
CMyListCtrl::CMyListCtrl(void)
{
	m_bkdImage = NULL;
	m_PaneInfo.nCurDir = 0;
	m_PaneInfo.nParentDir = 0;
	m_PaneInfo.nCurItem = 0;
	m_strCurrPath = _T("/"); // .
	m_bOverflow = FALSE;
	m_bCSV = FALSE;
}


CMyListCtrl::~CMyListCtrl(void)
{
	ClearImgVector();
}

void CMyListCtrl::ClearImgVector(void)
{
	if (m_bkdImage)
	{
		delete m_bkdImage;
	}
	while (!m_vpImages.empty())
	{
		CBKDImage *bkdi = m_vpImages.back();
		m_vpImages.pop_back();
		if (bkdi)
			delete bkdi;
	}
	m_strCurrPath = _T("/"); //    
	m_strImageName = _T("");
}

void CMyListCtrl::PushCurrentImg(void)
{
	m_vpImages.push_back(m_bkdImage);
}

BOOL CMyListCtrl::PopCurrentImg(void)
{
	if (m_vpImages.empty())
		return FALSE;
	if (m_bkdImage)
	{
		delete m_bkdImage;
	}
	m_bkdImage = m_vpImages.back();
	m_vpImages.pop_back();

	return TRUE;
}

BOOL CMyListCtrl::SetImgName(const CString& strName, long nBaseOffset, ULONGLONG nSize, IMAGE_TYPES type, BOOL bLogDisk)
{
	TCHAR pDrive[_MAX_DRIVE];
	TCHAR pDir[_MAX_DIR];
	TCHAR pName[_MAX_FNAME];
	TCHAR pExt[_MAX_EXT];

	// ,        .
	m_strOSName = GetImageFormat(type);
	//   .
	if (type != IMAGE_TYPE_ERROR && type != IMAGE_TYPE_UNKNOWN)
	{
		_tsplitpath_s (strName, pDrive, _MAX_DRIVE, pDir, _MAX_DIR, pName, _MAX_FNAME, pExt, _MAX_EXT);
		m_strImgPath = CString(pDrive) + CString(pDir);
		m_strImageName = CString(pName) + CString(pExt);
		//       
		switch (type)
		{
		case IMAGE_TYPE_ANDOS:
			m_bkdImage = new CBKDAndos(this, nBaseOffset, strName);
			break;
		case IMAGE_TYPE_DXDOS:
			//    . (  ,
			//    12    )
			//   ,          
			m_bkdImage = new CBKDAndos(this, nBaseOffset, strName);
			break;
		case IMAGE_TYPE_MKDOS:
			m_bkdImage = new CBKDMkdos(this, nBaseOffset, strName);
			break;
		case IMAGE_TYPE_AODOS:
			m_bkdImage = new CBKDAodos(this, nBaseOffset, strName);
			break;
		case IMAGE_TYPE_NORD:
			m_bkdImage = new CBKDNord(this, nBaseOffset, strName);
			break;
		case IMAGE_TYPE_MIKRODOS:
			m_bkdImage = new CBKDMikrodos(this, nBaseOffset, strName);
			break;
		case IMAGE_TYPE_CSIDOS3:
			m_bkdImage = new CBKDCsidos3(this, nBaseOffset, strName);
			break;
		case IMAGE_TYPE_RT11:
			m_bkdImage = new CBKDRT11(this, nBaseOffset, strName);
			break;
		case IMAGE_TYPE_NCDOS:
			m_bkdImage = new CBKDNCdos(this, nBaseOffset, strName);
			break;
		default:
			ASSERT(FALSE);
		}
		m_bkdImage->SetImgSize(nSize);
		m_bkdImage->SetImgType(type);
		// 
		m_bkdImage->ReadCatalog();
		// ,   .
		if (bLogDisk)
		{
			m_vSelItems.push_back(m_PaneInfo);
			m_PaneInfo.nCurDir = 0;
			m_PaneInfo.nParentDir = 0;
			m_PaneInfo.nCurItem = 0;
		}
		else
		{
			m_PaneInfo.nCurDir = 0;
			m_PaneInfo.nParentDir = 0;
			m_PaneInfo.nCurItem = 0;
			m_vSelItems.clear();
			m_vSelItems.push_back(m_PaneInfo);
		}
		return TRUE;
	}
	return FALSE;
}

BOOL CMyListCtrl::ResetImgName()
{
	//   .
	m_bkdImage->Reset();
	// 
	m_bkdImage->ReadCatalog();
	//   .
	m_PaneInfo = m_vSelItems.back();
	m_vSelItems.pop_back();
	return TRUE;
}

void CMyListCtrl::CloseImage(void)
{
	if (m_bkdImage)
	{
		delete m_bkdImage;
		m_bkdImage = NULL;
	}
	m_PaneInfo.nCurDir = 0;
	m_PaneInfo.nParentDir = 0;
	m_PaneInfo.nCurItem = 0;
	m_vSelItems.clear();
	m_vSelItems.push_back(m_PaneInfo);
	m_strCurrPath = _T("/"); // .
}

void CMyListCtrl::SetCSVParams(BOOL bCSV)
{
	m_bCSV = bCSV;
}

void CMyListCtrl::OutDir(int iCurDir)
{
	if (m_bOverflow)
		return;
	m_nRecursionCount++;
	if (m_nRecursionCount < 32)
	{
		FileRecord cur_fr;
		list<BKDir> l_Dirs;
		BKDir dir;
		if (!m_bCSV)
		{
			_ftprintf(file_lst, _T("[+]%s%s\n"), m_strImageName, m_strCurrPath);
		}
		_ftprintf(file_txt, _T("[+]%s%s\n"), m_strImageName, m_strCurrPath);
		BOOL bEnd;
		if (!(bEnd = m_bkdImage->GetStartFileName(&cur_fr, iCurDir)))
		{
			_ftprintf(file_lst, _T("No files in current directory!\n"));
			_ftprintf(file_txt, _T("No files in current directory!\n"));
		}
		else
		{
			do 
			{
				//  
				if (cur_fr.nAttr & FR_ATTR_DIR)
				{
					if (cur_fr.strName != _T(".") && cur_fr.strName != _T(".."))
					{
						if (m_strCurrPath == _T("/"))
						{
							dir.name = m_strCurrPath + cur_fr.strName;
						}
						else
						{
							dir.name = m_strCurrPath + _T("/") + cur_fr.strName;
						}
						dir.num = cur_fr.iDirDir; //   0     
						dir.bLogDisk = FALSE;
						if (dir.num) l_Dirs.push_back(dir);
					}
				}
				else
				{
					if (cur_fr.nAttr&FR_ATTR_LOGDISK)
					{
						if (m_strCurrPath == _T("/"))
						{
							dir.name = m_strCurrPath + cur_fr.strName;
						}
						else
						{
							dir.name = m_strCurrPath + _T("/") + cur_fr.strName;
						}
						dir.num = 0;
						dir.bLogDisk = TRUE;
						dir.nBaseOffset = (m_bkdImage->GetBaseOffset() + cur_fr.iStartSector * 512);
						dir.nSize = cur_fr.iSize;
						l_Dirs.push_back(dir);
					}
					else
					{
						if (cur_fr.nAttr&FR_ATTR_DELETED || cur_fr.nAttr&FR_ATTR_BAD)
						{
							//       .
						}
						else
						{
							if (m_bCSV)
							{
								_ftprintf(file_lst, _T("%s; %s; %s; %s; %s; %s; %s; %04d; %06o; %06o\n"),
									m_strImgPath,
									m_strImageName,
									m_strOSName,
									(g_bImageBootable ? _T("Yes"):_T("No")),
									(m_bkdImage->GetBaseOffset() != 0 ? _T("Yes"):_T("No")),
									m_strCurrPath,
									cur_fr.strName,
									cur_fr.iBlkSize,
									cur_fr.iAddress,
									cur_fr.iSize
									);
							}
							else
							{
								_ftprintf(file_lst, _T("   %-14s = Blk: %04d ; Addr: %06o ; Size: %06o\n"),
									cur_fr.strName,
									cur_fr.iBlkSize,
									cur_fr.iAddress,
									cur_fr.iSize
									);
							}
							_ftprintf(file_txt, _T("\t%s\n"), cur_fr.strName );
						}
					}
				}
				bEnd = m_bkdImage->GetNextFileName(&cur_fr, iCurDir);
			} while (bEnd);
			if (!m_bCSV)
			{
				_ftprintf(file_lst, _T("\n"));
			}
			_ftprintf(file_txt, _T("\n"));
			//     ,     
			while (!l_Dirs.empty())
			{
				dir = l_Dirs.front();
				m_strCurrPath = dir.name;
				if (dir.bLogDisk)
				{
					CloseCurrentImage();
					IMAGE_TYPES iImageOSType = ParseImage(dir.nBaseOffset);
					if (iImageOSType == IMAGE_TYPE_ERROR)
					{
						if (m_bCSV)
						{
							_ftprintf(file_lst, _T("%s%s Logical Disk - IO Error!\n"),m_strImageName, m_strCurrPath);
						}
						else
						{
							_ftprintf(file_lst, _T("[*]%s%s Logical Disk - IO Error!\n"),m_strImageName, m_strCurrPath);
						}
					}
					else
					{
						if (!m_bCSV)
						{
							CString strFormat = GetImageFormat(iImageOSType);
							_ftprintf(file_lst, _T("[*]%s%s Logical Disk = Size: %d; Format: %s\n"),m_strImageName, m_strCurrPath, dir.nSize, strFormat);
						}
						if (iImageOSType != IMAGE_TYPE_UNKNOWN)
						{
							//,   ,   ,     
							PushCurrentImg();
							if (SetImgName(g_strImgPathName, dir.nBaseOffset, dir.nSize, iImageOSType, TRUE))
							{
								SetCSVParams(m_bCSV);
								OutDir(0);
							}
							CloseCurrentImage();
							PopCurrentImg();
							ResetImgName();
						}
					}
				}
				else
				{
					OutDir(dir.num);
				}
				l_Dirs.pop_front();
			}
		}
	}
	else
	{
		_ftprintf(file_lst, _T("Recursion overflow!\n"));
		_ftprintf(file_txt, _T("Recursion overflow!\n"));
		m_bOverflow = TRUE;
	}
	m_nRecursionCount--;
}
