#include "StdAfx.h"
#include "BKDAndos.h"


CBKDAndos::CBKDAndos(CMyListCtrl* pLC, long nBaseOffset, CString strFileName): CBKDImage(pLC, nBaseOffset, strFileName)
{
	m_nSectorSize = 0;
	m_nClusterSectors = 0;
	m_nClusterSize = 0;
	m_nBootSectors = 0;
	m_nRootFilesNum = 0;
	m_nRootSize = 0;
	m_nFatSectors = 0;
	m_nFatSize = 0;
	m_pFatTbl = NULL;
}


CBKDAndos::~CBKDAndos(void)
{
	if (m_pFatTbl)
		delete m_pFatTbl;
}

BOOL CBKDAndos::ReadCatalog()
{
	CBKDImage::ReadCatalog();

	if(!m_bFileOpened)
	{
		return FALSE;
	}
	AndosFileRecord curr_record;
	FileRecord curr_fr;
	CString ext;

	//  
	g_ImgFile.Seek(m_nBaseOffset + 0, CFile::begin);
	if (g_ImgFile.Read (m_mSector, sizeof(m_mSector)) < sizeof(m_mSector))
	{
		return FALSE;
	}
	//  BPB
	m_nSectorSize = *(WORD*)&m_mSector[013];
	m_nClusterSectors = m_mSector[015];
	m_nClusterSize = m_nClusterSectors * m_nSectorSize;
	m_nBootSectors = *(WORD*)&m_mSector[016];
	m_nRootFilesNum = *(WORD*)&m_mSector[021];
	m_nRootSize = m_nRootFilesNum * 32;
	m_nFatSectors = *(WORD*)&m_mSector[026];
	m_nFatSize = m_nFatSectors * m_nSectorSize;

	m_pFatTbl = new BYTE[m_nFatSize];
	g_ImgFile.Seek(m_nBaseOffset + m_nBootSectors * m_nSectorSize, CFile::begin); //   
	//   ,  2   
	if ((int)g_ImgFile.Read (m_pFatTbl, m_nFatSize) < m_nFatSize)
	{
		return FALSE;
	}

	m_nRootSectorOffset = (m_nBootSectors + m_nFatSectors * 2) * m_nSectorSize;
	m_nDataSectorOffset = m_nRootSectorOffset + m_nRootSize;
	//   
	if (m_nBaseOffset + m_nRootSectorOffset != g_ImgFile.Seek(m_nBaseOffset + m_nRootSectorOffset, CFile::begin))
	{
		return FALSE;
	}

	//  7 .   32. ,  -  112.
	int rec_count = m_nRootFilesNum;
	int free_recs = m_nRootFilesNum;
	int used_size = 0;
	while (--rec_count >= 0)
	{
		if (g_ImgFile.Read (&curr_record, sizeof(AndosFileRecord)) < sizeof(AndosFileRecord))
		{
			return FALSE; //    ,    ,   ,
			//    .
		}
		//  ,       
		if (curr_record.name[0] == 0)
		{
			break; //    
		}
		//  , 0 -    ,  -     ,
		//         .
		//       ,      .
		curr_fr.nAttr = 0;
		if (curr_record.name[0] == 0345)
		{
			curr_fr.nAttr |= FR_ATTR_DELETED;
			free_recs++;
		}
		free_recs--;
		if (curr_record.dir_num)
		{ // 
			curr_fr.nAttr |= FR_ATTR_DIR;
			curr_fr.nRecType = RECORD_TYPE_DIR;
			curr_fr.strName = BKToUNICODE((BYTE*)curr_record.name, 8).Trim() + BKToUNICODE((BYTE*)curr_record.ext, 3).Trim();
			curr_fr.iDirDir = curr_record.dir_num;
			curr_fr.iDirNum = curr_record.parent_dir_num;
			curr_fr.iBlkSize = 0;
		}
		else
		{ // 
			curr_fr.nRecType = RECORD_TYPE_FILE;
			curr_fr.strName = BKToUNICODE((BYTE*)curr_record.name, 8).Trim();
			ext = BKToUNICODE((BYTE*)curr_record.ext, 3).Trim();
			if (!ext.IsEmpty())
			{
				curr_fr.strName += _T(".") + ext;
			}
			curr_fr.iDirDir = 0;
			curr_fr.iDirNum = curr_record.parent_dir_num;
			//    .
			curr_fr.iBlkSize = (curr_record.length) ? ((((curr_record.length - 1) | (m_nClusterSize - 1)) + 1) / m_nClusterSize) : 0;
			if (!(curr_fr.nAttr & FR_ATTR_DELETED)) used_size += curr_fr.iBlkSize;
		}
		curr_fr.nSpecificData = curr_record.date;
		curr_fr.iAddress = curr_record.address;
		curr_fr.iSize = curr_record.length;
		curr_fr.iStartSector = curr_record.first_cluster;
		m_sDiskCat.vecFC.push_back(curr_fr);
	}
	m_sDiskCat.nFreeRecs = free_recs; //    
	m_sDiskCat.nFreeBlocks = (*(WORD*)&m_mSector[023] - m_nDataSectorOffset / m_nSectorSize) / m_nClusterSectors - used_size;
	return TRUE;
}


CString CBKDAndos::GetInfo()
{
	CString str, strf;
	//strf.LoadString(IDS_INFO_FREE_CLUS);
	int bytes = m_sDiskCat.nFreeBlocks * m_nClusterSize;
	str.Format(strf, m_sDiskCat.nFreeRecs, tblStrRec[GetWordEndIdx(m_sDiskCat.nFreeRecs)], m_sDiskCat.nFreeBlocks, tblStrBlk[GetWordEndIdx(m_sDiskCat.nFreeBlocks)], bytes, tblStrBlk[GetWordEndIdx(bytes)]);
	return str;
}

int CBKDAndos::GetNextFat(int fat)
{
/*
0  1  2  3  4  5  6  7  8  9
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 
0                   1 
fat*3/2 
0 1 2 3 4 5 6 7  8
0 1 3 4 6 7 9 10 12
*/
	int num = fat * 3 / 2;
	int val = *(WORD*)&m_pFatTbl[num];
	if (fat & 1)
	{ //  ,   12 
		val >>= 4;
	}
	//  ,   12 
	return (val & 0xfff);
}

//     (   )
int CBKDAndos::FindFreeFat(int fat)
{
	WORD val;
	int num = fat * 3 / 2;
	while (num < (m_nFatSize - 1))
	{
		val = *(WORD*)&m_pFatTbl[num];
		if (fat & 1)
		{ //  ,   12 
			val >>= 4;
		}
		val &= 0xfff;

		if (val == 0)
		{ //  
			return fat;
		}
		fat++;
		num = fat * 3 / 2;
	}
	return -1;
}
//   .
void CBKDAndos::SetFat(int fat, WORD val)
{
	int num = fat * 3 / 2;
	WORD cval = *(WORD*)&m_pFatTbl[num];
	val &= 0xfff;
	if (fat & 1)
	{
		*(WORD*)&m_pFatTbl[num] = (cval & 0x000f) | (val << 4);
	}
	else
	{
		*(WORD*)&m_pFatTbl[num] = (cval & 0xf000) | val;
	}
}

void CBKDAndos::ExtractFile(FileRecord *sFile, BOOL bExtractAsBin)
{
	CFile outFile;

	CString fname = sFile->strName;
	SetSafeName(fname);

	if (outFile.Open (m_strStorePath + fname + ((bExtractAsBin) ? _T(".bin") : _T("")), CFile::modeCreate|CFile::modeWrite))
	{
		m_currFat = sFile->iStartSector;
		int len = sFile->iSize;
		int out_len;
		if (bExtractAsBin)
		{
			m_mBinHeader[0] = (WORD)sFile->iAddress;
			m_mBinHeader[1] = (WORD)sFile->iSize; //  ,    64,  .
			outFile.Write(m_mBinHeader, sizeof(m_mBinHeader));
		}
		while((len > 0) && (m_currFat < 07760))
		{
			SeekToCluster(&g_ImgFile);
			g_ImgFile.Read (&m_mBlock, m_nClusterSize);

			if (len >= m_nClusterSize)
			{
				out_len = m_nClusterSize;
			}
			else
			{
				out_len = len;
			}
			len -= out_len;
			outFile.Write(&m_mBlock, out_len);
			m_currFat = GetNextFat(m_currFat);
		}
		outFile.Close();
	}
}

void CBKDAndos::SeekToCluster(CFile *f)
{
	m_nLenClust = m_nClusterSize;
	if (m_currFat < 07760)
	{
		int sector = m_nDataSectorOffset + (m_currFat - 2) * m_nClusterSize;
		m_tPos = f->Seek(m_nBaseOffset + sector, CFile::begin);
	}
}

int CBKDAndos::SeekToPos(CFile *f, int pos)
{
	//     ,    
	while (pos > m_nClusterSize)
	{
		pos -= m_nClusterSize;
		m_currFat = GetNextFat(m_currFat);
		m_tLen -= m_nClusterSize;
	}
	SeekToCluster(f);
	//    
	g_ImgFile.Seek( m_tPos + pos, CFile::begin);
	m_nLenClust -= pos;
	return m_tLen - pos;
}

void CBKDAndos::SeekToBeginFile(CFile *f, FileRecord *fr)
{
	m_currFat = fr->iStartSector;
	SeekToCluster(f);
}

BYTE CBKDAndos::GetByte(CFile *f, int *len)
{
	BYTE b;
	f->Read(&b, sizeof(b));
	*len -= sizeof(b);
	m_nLenClust -= sizeof(b); //    
	if (m_nLenClust <= 0)
	{ //    
		m_currFat = GetNextFat(m_currFat);
		SeekToCluster(f);
	}
	return b;
}

WORD CBKDAndos::GetWord(CFile *f, int *len)
{
	WORD w;
	f->Read (&w, sizeof(w));
	*len -= sizeof(w);
	m_nLenClust -= sizeof(w); //    
	if (m_nLenClust <= 0)
	{ //    
		m_currFat = GetNextFat(m_currFat);
		SeekToCluster(f);
	}
	return w;
}

FILESAVE_STATUS CBKDAndos::AddFile(CString& str, int nDirNum, BOOL bAddAsBin)
{
	if (m_bFileROMode)
	{ //     ,
		return FS_IMAGE_RO; //       .
	}
	CFile inFile;
	FILESAVE_STATUS bRet = FS_IO_ERROR;
	int FileAddr, FileSize;

	UINT CatOffs = 0; //   ,     .
	AndosFileRecord aFR;
	ZeroMemory(&aFR, sizeof(AndosFileRecord));

	if (inFile.Open (str, CFile::modeRead | CFile::shareDenyWrite))
	{
		FileSize = (int)inFile.GetLength();
		CString name = inFile.GetFileName();
		CString ext = _T("");
		int l = name.GetLength();
		int t = name.ReverseFind(_T('.'));
		if (t >= 0) //  
		{
			ext = name.Right(l - t - 1);
			name = name.Left(t);
		}
		//,      ?
		inFile.Read(m_mBinHeader, sizeof(m_mBinHeader));
		//  ,    "  "   
		if (((ext.CompareNoCase(_T("BIN"))== 0) || bAddAsBin) && (m_mBinHeader[1] == FileSize - 4)) //    
		{ //,  
			FileAddr = m_mBinHeader[0];
			FileSize = m_mBinHeader[1];
			//      .bin
			t = name.ReverseFind(_T('.')); //,    
			if (t >= 0)
			{
				l = name.GetLength();
				ext = name.Right(l - t - 1);
				name = name.Left(t);
			}
		}
		else
		{ //,  
			inFile.Seek(0, CFile::begin);
			FileAddr = 01000; // ,    
		}

		UNICODEtoBK(name, aFR.name, 8, TRUE);
		UNICODEtoBK(ext, aFR.ext, 3, TRUE);
		aFR.parent_dir_num = nDirNum; //   
		aFR.address = FileAddr; //,     ,    
		/*	 .  0-4 -   1-31; 
			 5-8   ,   1-12;
			 9-15  ,   1980 . ( MS-DOS),    0  127 , .. 1980-2107 .
		*/
		CFileStatus status;
		inFile.GetStatus(status);
		aFR.date = (status.m_ctime.GetDay() & 037) | ((status.m_ctime.GetMonth() & 017) << 5) | ((status.m_ctime.GetYear()-1980) << 9);
		aFR.length = FileSize;

		if (m_sDiskCat.nFreeBlocks * m_nClusterSize < ((FileSize) ? (((FileSize - 1) | (m_nClusterSize-1)) + 1) : 0))
		{
			inFile.Close();
			return FS_DISK_FULL;
		}
	}
	else
		return FS_FILE_OPENERR;

	AndosFileRecord* pCat = new AndosFileRecord[m_nRootSize / sizeof(AndosFileRecord)];
	ZeroMemory(pCat, m_nRootSize);

	/*
	1.    .
	2.     
	3.  , ,      
	4.   ,   
	5.    ,   
	6.    .
	*/

	//  1 
	if (m_nBaseOffset + m_nBootSectors*m_nSectorSize == g_ImgFile.Seek(m_nBaseOffset + m_nBootSectors*m_nSectorSize, CFile::begin))
	{
		// 
		if (m_nFatSize == g_ImgFile.Read (m_pFatTbl, m_nFatSize))
		{
			//  5 
			if (m_nBaseOffset + m_nRootSectorOffset == g_ImgFile.Seek(m_nBaseOffset + m_nRootSectorOffset, CFile::begin))
			{
				// 
				if (m_nRootSize == g_ImgFile.Read (pCat, m_nRootSize))
				{
					//,      .
					UINT tc = 0;
					BOOL bFound = FALSE;
					while (tc < m_nRootSize / sizeof(AndosFileRecord))
					{
						if (pCat[tc].name[0] == 0)
							break; // 

						if (!strncmp((char*)aFR.name, (char*)pCat[tc].name, 11))
						{
							bFound = TRUE;
							break;
						}
						tc++;
					}
					if (bFound)
					{
						bRet = FS_FILE_EXIST;
					}
					else
					{
						//    .
						CatOffs = 0;
						bFound = FALSE;
						while (CatOffs < m_nRootSize / sizeof(AndosFileRecord))
						{
							if ((pCat[CatOffs].name[0] == 0345) || (pCat[CatOffs].name[0] == 0))
							{
								bFound = TRUE;
								break;
							}
							CatOffs++;
						}

						if (bFound)
						{
							// 
							m_currFat = 1;
							BOOL bFirstFat = TRUE;
							BOOL bErr = FALSE;
							int nPrevFat = 0;
							do {
								m_currFat = FindFreeFat(m_currFat+1); //   ,     
								//     , ..         
								if (m_currFat == -1)
								{
									bErr = TRUE;
									break;
								}
								if (bFirstFat)
								{
									bFirstFat = FALSE;
									aFR.first_cluster = m_currFat;
								}
								else
								{
									SetFat(nPrevFat, m_currFat);
								}
								nPrevFat = m_currFat;
								SeekToCluster(&g_ImgFile);
								ZeroMemory(m_mBlock, m_nClusterSize); //         ,      
								inFile.Read(m_mBlock, m_nClusterSize); //    .  .
								g_ImgFile.Write(m_mBlock, m_nClusterSize); //   .
								FileSize -= m_nClusterSize;
							} while(FileSize > 0);
							if (bErr)
								bRet = FS_STRUCT_ERR;
							else
							{
								SetFat(nPrevFat, 07777);
								bRet = FS_NO_ERROR;
							}
						}
						else
							bRet = FS_CAT_FULL;
					}
				}
			}
		}
	}
	inFile.Close();
	if (bRet == FS_NO_ERROR)
	{ //   ,  
		g_ImgFile.Seek(m_nBaseOffset + m_nBootSectors*m_nSectorSize, CFile::begin); //   
		g_ImgFile.Write(m_pFatTbl, m_nFatSize); //   
		g_ImgFile.Write(m_pFatTbl, m_nFatSize); //   
		pCat[CatOffs] = aFR;
		g_ImgFile.Write(pCat, m_nRootSize); // 
	}
	delete [] pCat;
	return bRet;
}

CString CBKDAndos::GetSpecificData(DWORD dd)
{
	CString str;
	if (dd == 0)
		str = _T("");
	else
	{
		int nDay = dd & 0x1f;
		int nMon = (dd >> 5) & 0xf;
		int nYear = ((dd >> 9) & 0x7f) + 1980;
		str.Format(_T("%02d.%02d.%04d"), nDay, nMon, nYear);
	}
	return str;
}
