#include "StdAfx.h"
#include "BKDImage.h"
#include "MyListCtrl.h"
//#include "SprWnd.h"

CBKDImage::CBKDImage(CMyListCtrl* pLC, long nBaseOffset, CString strFileName)
{
	m_ParentLisctCtrl = pLC;
	m_sDiskCat.nFreeRecs = 0;
	m_sDiskCat.nFreeBlocks = 0;
	m_nBaseOffset = nBaseOffset;

	m_bFileOpened = FALSE;
	m_bFileROMode = FALSE;
	m_strImgPathName = strFileName;
	m_ImgType = IMAGE_TYPE_UNKNOWN;
	Reset();
}

CBKDImage::~CBKDImage(void)
{
	CloseFile();
}


void CBKDImage::Reset()
{
	OpenFile();
}


BOOL CBKDImage::OpenFile(void)
{
	CloseFile();

	if (g_ImgFile.Open (m_strImgPathName, CFile::modeReadWrite | CFile::shareDenyWrite))
	{
		m_bFileOpened = TRUE;
		m_bFileROMode = FALSE;
	}
	else if (g_ImgFile.Open (m_strImgPathName, CFile::modeRead | CFile::shareDenyWrite))
	{
		m_bFileOpened = TRUE;
		m_bFileROMode = TRUE;
	}
	return m_bFileOpened;
}


void CBKDImage::CloseFile(void)
{
	if (m_bFileOpened)
	{
		g_ImgFile.Close();
		m_bFileOpened = FALSE;
	}
}
// ,    -  .
BOOL CBKDImage::ReadCatalog()
{
	m_sDiskCat.vecFC.clear();
	m_sDiskCat.nVectorPos = 0;

	return TRUE;
}
//        num
int CBKDImage::GetWordEndIdx(int num)
{
	num %= 100; //   2   , -  10..19
	//
	if ((num >= 10) && (num <= 19))
	{
		return 2;
	}
	//        .
	//    
	int dg = num % 10;
	if (dg == 0)
		return 2;
	else if (dg == 1)
		return 0;
	else if ((2 <= dg) && (dg <= 4))
		return 1;
	else
		return 2;
}

CString CBKDImage::GetInfo()
{
	CString str, strf;
	//strf.LoadString(IDS_INFO_FREE_BLK);
	int bytes = m_sDiskCat.nFreeBlocks * 512;
	str.Format(strf, m_sDiskCat.nFreeRecs, tblStrRec[GetWordEndIdx(m_sDiskCat.nFreeRecs)], m_sDiskCat.nFreeBlocks, tblStrBlk[GetWordEndIdx(m_sDiskCat.nFreeBlocks)], bytes, tblStrBlk[GetWordEndIdx(bytes)]);
	return str;
}

void CBKDImage::SetStorePath(const CString& str)
{
	m_strStorePath = str;
	m_strStorePath.TrimRight(_T('\\'));
	m_strStorePath += _T('\\');
}
//      +    '\'  '/'   '.'
void CBKDImage::SetSafeName(CString& str)
{
	SetSafeDir(str);
	str.Replace(_T('/'), _T('.'));
	str.Replace(_T('\\'), _T('.'));
}
//   '>','<',':','|','?','*'   '_'
//     -  '_'
void CBKDImage::SetSafeDir(CString& str)
{
	str.Replace(_T('>'), _T('_'));
	str.Replace(_T('<'), _T('_'));
	str.Replace(_T(':'), _T('_'));
	str.Replace(_T('|'), _T('_'));
	str.Replace(_T('?'), _T('_'));
	str.Replace(_T('*'), _T('_'));
	//        32
	for(int i = 0; i < str.GetLength(); i++)
	{
		if (str.GetAt(i) < 32)
		{
			str.SetAt(i, _T('_'));
		}
	}
}


void CBKDImage::ExtractObject(FileRecord *sFile, BOOL bExtractAsBin)
{
	//      -   
	if (sFile->nAttr & (FR_ATTR_BAD | FR_ATTR_DELETED | FR_ATTR_LINK))
	{
		return;
	}
	if (sFile->nAttr & FR_ATTR_DIR)
	{ // ,         .
		int cur_dir = sFile->iDirDir;
		FileRecord cur_fr;
		CString tmpStorePath = m_strStorePath;
		CString strDirName = sFile->strName;
		SetSafeDir(strDirName);
		SetStorePath(m_strStorePath + strDirName);
		_tmkdir(m_strStorePath);

		BOOL bEnd = GetStartFileName(&cur_fr, cur_dir);
		if (bEnd)
		{ //      
			if (cur_fr.strName != _T(".."))
			{
				UINT tmpNvpos = m_sDiskCat.nVectorPos; //  ,       
				ExtractObject(&cur_fr, bExtractAsBin);
				m_sDiskCat.nVectorPos = tmpNvpos;
			}
		}
		while (bEnd) //  
		{
			bEnd = GetNextFileName(&cur_fr, cur_dir);
			if (bEnd)
			{
				UINT tmpNvpos = m_sDiskCat.nVectorPos; //  ,       
				ExtractObject(&cur_fr, bExtractAsBin);
				m_sDiskCat.nVectorPos = tmpNvpos;
			}
		}
		m_strStorePath = tmpStorePath;
	}
	else
	{ // 
		ExtractFile(sFile, bExtractAsBin);
	}
}

//      ,      
void CBKDImage::ExtractFile(FileRecord *sFile, BOOL bExtractAsBin)
{
	//      -   
	if (sFile->nAttr & (FR_ATTR_BAD | FR_ATTR_DELETED | FR_ATTR_DIR))
	{
		return;
	}

	CFile outFile;

	CString fname = sFile->strName;
	SetSafeName(fname);

	if (outFile.Open (m_strStorePath + fname + ((bExtractAsBin)?_T(".bin") : _T("")), CFile::modeCreate | CFile::shareExclusive | CFile::modeWrite))
	{
		int len = sFile->iSize;
		int out_len;
		int sector = sFile->iStartSector;
		g_ImgFile.Seek(m_nBaseOffset + sector * 512, CFile::begin); //   
		//    ,    
		if (bExtractAsBin)
		{
			m_mBinHeader[0] = (WORD)sFile->iAddress;
			m_mBinHeader[1] = (WORD)sFile->iSize; //  ,    64,  .
			outFile.Write(m_mBinHeader, sizeof(m_mBinHeader));
		}
		while(len > 0)
		{
			//     ,   
			g_ImgFile.Read (&m_mBlock, COPY_BLOCK_SIZE);

			if (len >= COPY_BLOCK_SIZE)
			{
				out_len = COPY_BLOCK_SIZE;
			}
			else
			{
				out_len = len;
			}
			outFile.Write(&m_mBlock, out_len);
			len -= out_len;
		}

		outFile.Close();
	}
}

//    
void CBKDImage::ViewFile(FileRecord *sFile)
{
	//    -    
	if (sFile->nAttr & (FR_ATTR_BAD | FR_ATTR_DELETED | FR_ATTR_DIR | FR_ATTR_LOGDISK))
	{
		return;
	}

	TCHAR pTmpPath[MAX_PATH];
	CFile tmpFile;
	CString strFName = sFile->strName;
	SetSafeName(strFName);

	_tsplitpath_s(strFName, NULL, 0, NULL, 0, NULL, 0, pTmpPath, MAX_PATH); // 
	CString strExt = CString(pTmpPath).MakeUpper();

	GetTempPath(MAX_PATH, pTmpPath);
	strFName = CString(pTmpPath) + strFName;

	if (tmpFile.Open (strFName, CFile::modeCreate | CFile::modeWrite | CFile::shareExclusive))
	{
		int len = sFile->iSize;
		SeekToBeginFile(&g_ImgFile, sFile); //   
		//    ,    ,
		//      
		BYTE b;
		int counter = 0;
		TCHAR tch = 0xfeff; //BOM
		if ((strExt == _T(".VXT")) || (strExt == _T(".EDP")))
		{
			PutTchar(&tmpFile, tch, &counter); //  BOM

			SaveCurrPos(&g_ImgFile, len);

			if (GetWord(&g_ImgFile, &len) == 0x0c40)
			{ //      ..
				counter = GetWord(&g_ImgFile, &len);
			}
			else
				counter = 0;
			//       
			len = SeekToPos(&g_ImgFile, counter);

			int tabs = 0;
			counter = 0;
			while (len > 0)
			{
				b = GetByte(&g_ImgFile, &len);
				if (len < 0) //.   
					break;

				if (b == 0 || b == 10) // 
				{
					PutTchar(&tmpFile, _T('\r'), &counter);
					counter = -1;
					tch = _T('\n');
				}
				else if (b < 9) //    ,     .
				{
					if (tabs == 0)
						tabs++;
					for(int i = 0; i < b; i++)
					{
						for(int j = 0; j < tabs; j++)
						{
							PutTchar(&tmpFile, _T(' '), &counter);
						}
					}
					tabs <<= 3;
					continue;
				}
				else if (b == 9) //,  8   
				{
					tabs = 0;
					do
					{
						PutTchar(&tmpFile, _T(' '), &counter);
					} while ((counter & 7) != 0);
					continue;
				}
				else if (b < 32)
				{
					tabs = 0;
					continue; //   vxt    ,   ,   ..
				}
				else
				{
					tch = BKToUNICODE_Byte(b);
				}
				tabs = 0;
				PutTchar(&tmpFile, tch, &counter);
			}
		}
		else
		{
			PutTchar(&tmpFile, tch, &counter); //  BOM
			while(len > 0)
			{
				b = GetByte(&g_ImgFile, &len);
				if (b == 0 || b == 10) // 
				{
					PutTchar(&tmpFile, _T('\r'), &counter);
					tch = _T('\n');
				}
				else if (b == 9) //
				{
					tch = _T('\t');
				}
				else
				{
					tch = BKToUNICODE_Byte(b);
				}
				PutTchar(&tmpFile, tch, &counter);
			}
		}
		tmpFile.Close();

		//       
		STARTUPINFO si;
		PROCESS_INFORMATION pi;

		ZeroMemory( &si, sizeof(si) );
		si.cb = sizeof(si);
		ZeroMemory( &pi, sizeof(pi) );

		CString strCmdLine = _T("notepad \"") + strFName + _T("\"");
		if (CreateProcess(NULL, // No module name (use command line)
			strCmdLine.GetBuffer(), // Command line
			NULL,           // Process handle not inheritable
			NULL,           // Thread handle not inheritable
			FALSE,          // Set handle inheritance to FALSE
			0,              // No creation flags
			NULL,           // Use parent's environment block
			NULL,           // Use parent's starting directory 
			&si,            // Pointer to STARTUPINFO structure
			&pi ))           // Pointer to PROCESS_INFORMATION structure
		{
			// Wait until child process exits.
			WaitForSingleObject( pi.hProcess, INFINITE );

			// Close process and thread handles. 
			CloseHandle( pi.hProcess );
			CloseHandle( pi.hThread );
		}
		//   
		CFile::Remove(strFName);
	}
	/*
	 :         ,
	 .vxt, .edp     ,
	   -    8  
	 ,    ,      
	 .    .
	*/
}

// void CBKDImage::ViewFileAsSprite(FileRecord *sFile)
// {
// 	//    -    
// 	if (sFile->nAttr & (FR_ATTR_BAD | FR_ATTR_DELETED | FR_ATTR_DIR | FR_ATTR_LOGDISK))
// 	{
// 		return;
// 	}
// 	int len = sFile->iSize;
// 	if (len > 0)
// 	{
// 		BYTE *pBuffer = new BYTE[len];
// 		if (pBuffer)
// 		{
// 			int l = len;
// 			int t = 0;
// 			SeekToBeginFile(&g_ImgFile, sFile);
// 			while(l > 0)
// 			{
// 				pBuffer[t++] = GetByte(&g_ImgFile, &l);
// 			}
// 
// 			CSprWnd *m_pSprWnd = new CSprWnd; //  .
// 			if (m_pSprWnd)
// 			{
// 				m_pSprWnd->SetBuffer(pBuffer, len);
// 				WNDCLASS wc;
// 				memset(&wc, 0, sizeof(wc));
// 				wc.style = CS_HREDRAW | CS_VREDRAW;
// 				wc.hInstance=AfxGetInstanceHandle();
// 				wc.lpfnWndProc=::DefWindowProc;
// 				wc.lpszClassName = _T("MyClass");
// 				wc.hbrBackground=(HBRUSH)::GetStockObject(GRAY_BRUSH);
// 				AfxRegisterClass(&wc);
// 				if (m_pSprWnd->Create(
// 					_T("MyClass"),		//  
// 					_T("SpriteView: ") + sFile->strName,	// 
// 					WS_POPUP | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME,	// window styles
// 					CRect(0,0,200,200),		// 
// 					theApp.GetMainWnd(),	// 
// 					0,
// 					WS_EX_TOOLWINDOW /*| WS_EX_OVERLAPPEDWINDOW*/	//extended window styles
// 					))
// 				{
// 					m_pSprWnd->ShowWindow(SW_SHOW);
// 				}
// 				// pBuffer   CSprWnd.     ,     ,
// 				//     pBuffer.
// 			}
// 			else
// 			{
// 				delete [] pBuffer;
// 			}
// 		}
// 	}
// }


BOOL CBKDImage::GetStartFileName(FileRecord *sFile, int iDirNum)
{
	m_sDiskCat.nVectorPos = 0; // -       ,   .
	//   ,   ..
	//     .. -      .
	sFile->clear();
	sFile->strName = _T("..");
	sFile->nAttr = FR_ATTR_DIR;
	sFile->nRecType = RECORD_TYPE_UP;
	sFile->iDirNum = iDirNum;
	return TRUE;
}


BOOL CBKDImage::GetNextFileName(FileRecord *sFile, int iDirNum)
{
	sFile->clear();
	//   ,    ,   
	if (m_sDiskCat.nVectorPos>=m_sDiskCat.vecFC.size())
		return FALSE;
	//     .   .
	while (m_sDiskCat.nVectorPos < m_sDiskCat.vecFC.size())
	{
		if (m_sDiskCat.vecFC[m_sDiskCat.nVectorPos].iDirNum == iDirNum)
		{ //  
			*sFile = m_sDiskCat.vecFC[m_sDiskCat.nVectorPos];
			m_sDiskCat.nVectorPos++;
			return TRUE;
		}
		m_sDiskCat.nVectorPos++;
	}
	return FALSE;
}

/*
    8  .    
*/
CString CBKDImage::BKToUNICODE (BYTE* pBuff, int size)
{
	CString strRet = _T("");
	TCHAR tch;
	for (int i = 0; i < size; i++)
	{
		tch = BKToUNICODE_Byte(pBuff[i]);
		if (tch == 0)
			break;
		strRet += tch;
	}
	return strRet;
}

CString CBKDImage::BKToUNICODE11 (BYTE* pBuff, int size)
{
	CString strRet = _T("");
	TCHAR tch;
	for (int i = 0; i < size; i++)
	{
		tch = BKToUNICODE_Byte11(pBuff[i]);
		if (tch == 0)
			break;
		strRet += tch;
	}
	return strRet;
}

TCHAR CBKDImage::BKToUNICODE_Byte (BYTE b)
{
	if (b == 0)
		return 0;

	if (b < 32)
	{
		return _T(' ');
	}
	else if (b >= 32 && b < 127)
	{
		return (TCHAR)b;
	}
	else if (b == 127)
	{
		return (TCHAR)0x25a0;
	}
	else if (b >= 128)
	{
		return (TCHAR)koi8tbl[b - 128];
	}
	return 0; // return     .
}

TCHAR CBKDImage::BKToUNICODE_Byte11 (BYTE b)
{
	if (b == 0)
		return 0;

	if (b < 32)
	{
		return _T(' ');
	}
	else if (b >= 32 && b < 127)
	{
		return (TCHAR)b;
	}
	else if (b == 127)
	{
		return (TCHAR)0x25a0;
	}
	else if (b >= 128)
	{
		return (TCHAR)koi8tbl11[b - 128];
	}
	return 0;
}
/*
     8.
:
	ustr -  
	pBuff - ,   
	bufSize -  
	bFillBuf - .     ,     . (  - 0 .)
*/
void CBKDImage::UNICODEtoBK(CString& ustr, BYTE* pBuff, int bufSize, BOOL bFillBuf)
{
	int len = ustr.GetLength();
	int n = 0;
	int bn = 0;
	TCHAR ch;
	BYTE b;
	while (n < len)
	{
		ch = ustr.GetAt(n++); //  
		if (ch < 32) //   ,
			b = 32;  //  
		else if ((32 <= ch) && (ch < 127)) // --  
		{
			b = LOBYTE(ch); //  --  
		}
		else if (ch == 0x25a0) // 
		{
			b = 127; // .        
		}
		else //   
		{
			//      ,    -   8
			int i = 0;
			b = 32; //      -  
			while (i < 128)
			{
				if (koi8tbl[i] == ch)
				{
					b = LOBYTE(i) + 0200;
					break;
				}
				i++;
			}
		}
		pBuff[bn++] = b;
		if (bn >= bufSize)
		{
			break;
		}
	}
	if (bFillBuf && (len < bufSize))
	{
		while (bn < bufSize)
		{
			pBuff[bn++] = 32;
		}
	}
}


void CBKDImage::SaveCurrPos(CFile *f, int len)
{
	m_tPos = f->GetPosition();
	m_tLen = len;
}

int CBKDImage::SeekToPos(CFile *f, int pos)
{
	g_ImgFile.Seek( m_tPos + pos, CFile::begin);
	return m_tLen - pos;
}

void CBKDImage::SeekToBeginFile(CFile *f, FileRecord *fr)
{
	f->Seek(m_nBaseOffset + fr->iStartSector * 512, CFile::begin);
}

BYTE CBKDImage::GetByte(CFile *f, int *len)
{
	BYTE b;
	f->Read (&b, sizeof(b));
	*len -= sizeof(b);
	return b;
}

WORD CBKDImage::GetWord(CFile *f, int *len)
{
	WORD w;
	f->Read (&w, sizeof(w));
	*len -= sizeof(w);
	return w;
}

void CBKDImage::PutTchar(CFile *f, TCHAR tch, int *counter)
{
	f->Write(&tch, sizeof(tch));
	(*counter)++;
}

FILESAVE_STATUS CBKDImage::AddFile(CString& str, int nDirNum, BOOL bAddAsBin)
{
	return FS_IO_ERROR;
}
