﻿#pragma once
#include "globals.h"
#include <vector>
using namespace std;


#define FR_ATTR_PROTECTED 1
#define FR_ATTR_HIDDEN 2
#define FR_ATTR_DELETED 4
#define FR_ATTR_BAD 8
#define FR_ATTR_TEMPORARY 0x10
#define FR_ATTR_LOGDISK 0x20
#define FR_ATTR_LINK 0x40
#define FR_ATTR_DIR 0x80

#define RECORD_TYPE_UP 0
#define RECORD_TYPE_DIR 1
#define RECORD_TYPE_LINK 2
#define RECORD_TYPE_LOGDSK 3
#define RECORD_TYPE_FILE 4

#define COPY_BLOCK_SIZE 4096

struct FileRecord
{
	CString strName;	//имя
	UINT nAttr;			//атрибуты, ну там директория, защищённый, скрытый, удалённый, плохой
	int nRecType;		//тип записи: 0="..", 1=директория, 2=ссылка, 3=логический диск, 4=простой файл, вводится специально для сортировки
						//поэтому частично дублирует атрибуты
	int iDirNum;		//номер каталога, к которому принадлежит файл для файла и директории
	int iDirDir;		//номер директории для директории, 0 - для файла
	int iAddress;		//адрес файла, для директории 0, если для директории не 0, то это ссылка на другой диск.
	int iSize;			//размер файла
	int iBlkSize;		//размер в блоках, или кластерах (для андос)
	int iStartSector;	//начальный сектор/блок/кластер
	DWORD nSpecificData; //данные, индивидуальные для каждой ОС.
	FileRecord()
	{
		clear();
	}
	void clear(void)
	{
		strName = _T("");
		nAttr = 0;
		iStartSector = iBlkSize = iSize = iAddress = iDirDir = iDirNum = nRecType = 0;
		nSpecificData = 0;
	}
	FileRecord(const FileRecord *src)
	{
		strName = src->strName;
		nAttr = src->nAttr;
		nRecType = src->nRecType;
		iDirNum = src->iDirNum;
		iDirDir = src->iDirDir;
		iAddress = src->iAddress;
		iSize = src->iSize;
		iBlkSize = src->iBlkSize;
		iStartSector = src->iStartSector;
		nSpecificData = src->nSpecificData;
	}
	FileRecord & operator = (const FileRecord &src)
	{
		this->strName = src.strName;
		this->nAttr = src.nAttr;
		this->nRecType = src.nRecType;
		this->iDirNum = src.iDirNum;
		this->iDirDir = src.iDirDir;
		this->iAddress = src.iAddress;
		this->iSize = src.iSize;
		this->iBlkSize = src.iBlkSize;
		this->iStartSector = src.iStartSector;
		this->nSpecificData = src.nSpecificData;
		return *this;
	}
};


// это правильная таблица, но на бк всё по особому, даже кои8 там нестандартный
//таблица соответствия верхней половины аскии кодов с 128 по 255, включая псевдографику.
//выяснилось, что на бк11 таблица тоже нестандартная, но от бк10 тоже отличается.
//надо исправить потом.
static const TCHAR koi8tbl11[128] = {
	0x2500, 0x2502, 0x250C, 0x2510, 0x2514, 0x2518, 0x251C, 0x2524, 0x252C, 0x2534, 0x253C, 0x2580, 0x2584, 0x2588, 0x258C, 0x2590,
	0x2591, 0x2592, 0x2593, 0x2320, 0x25A0, 0x2219, 0x221A, 0x2248, 0x2264, 0x2265, 0xA0,   0x2321, 0xB0,   0xB2,   0xB7,   0xF7,
	0x2550, 0x2551, 0x2552, 0x451,  0x2553, 0x2554, 0x2555, 0x2556, 0x2557, 0x2558, 0x2559, 0x255A, 0x255B, 0x255C, 0x255D, 0x255E,
	0x255F, 0x2560, 0x2561, 0x401,  0x2562, 0x2563, 0x2564, 0x2565, 0x2566, 0x2567, 0x2568, 0x2569, 0x256A, 0x256B, 0x256C, 0xA9,
	_T('ю'), _T('а'), _T('б'), _T('ц'), _T('д'), _T('е'), _T('ф'), _T('г'), _T('х'), _T('и'), _T('й'), _T('к'), _T('л'), _T('м'), _T('н'), _T('о'),
	_T('п'), _T('я'), _T('р'), _T('с'), _T('т'), _T('у'), _T('ж'), _T('в'), _T('ь'), _T('ы'), _T('з'), _T('ш'), _T('э'), _T('щ'), _T('ч'), _T('ъ'),
	_T('Ю'), _T('А'), _T('Б'), _T('Ц'), _T('Д'), _T('Е'), _T('Ф'), _T('Г'), _T('Х'), _T('И'), _T('Й'), _T('К'), _T('Л'), _T('М'), _T('Н'), _T('О'),
	_T('П'), _T('Я'), _T('Р'), _T('С'), _T('Т'), _T('У'), _T('Ж'), _T('В'), _T('Ь'), _T('Ы'), _T('З'), _T('Ш'), _T('Э'), _T('Щ'), _T('Ч'), _T('Ъ')
};


//таблица соответствия верхней половины аскии кодов с 128 по 255, включая псевдографику
static const TCHAR koi8tbl[128] = { //{200..237} этих символов на бк нету.
	_T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '),
	_T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '),
	_T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '),
	_T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '), _T(' '),
	//{240..277}
	0xb6,    0x2534,  0x2665,  0x2510,  0x2561,  0x251c,  0x2514,  0x2550,
	0x2564,  0x2660,  0x250c,  0x252c,  0x2568,  0x2193,  0x253c,  0x2551,
	0x2524,  0x2190,  0x256c,  0x2191,  0x2663,  0x2500,  0x256b,  0x2502,
	0x2666,  0x2518,  0x256a,  0x2565,  0x2567,  0x255e,  0x2192,  0x2593,
	//{300..337}
	_T('ю'), _T('а'), _T('б'), _T('ц'), _T('д'), _T('е'), _T('ф'), _T('г'),
	_T('х'), _T('и'), _T('й'), _T('к'), _T('л'), _T('м'), _T('н'), _T('о'),
	_T('п'), _T('я'), _T('р'), _T('с'), _T('т'), _T('у'), _T('ж'), _T('в'),
	_T('ь'), _T('ы'), _T('з'), _T('ш'), _T('э'), _T('щ'), _T('ч'), _T('ъ'),
	//{340..377}
	_T('Ю'), _T('А'), _T('Б'), _T('Ц'), _T('Д'), _T('Е'), _T('Ф'), _T('Г'),
	_T('Х'), _T('И'), _T('Й'), _T('К'), _T('Л'), _T('М'), _T('Н'), _T('О'),
	_T('П'), _T('Я'), _T('Р'), _T('С'), _T('Т'), _T('У'), _T('Ж'), _T('В'),
	_T('Ь'), _T('Ы'), _T('З'), _T('Ш'), _T('Э'), _T('Щ'), _T('Ч'), _T('Ъ')
};

static const CString tblStrRec[3] = {
	_T("ь"), _T("и"), _T("ей")
};
static const CString tblStrBlk[3] = {
	_T(""), _T("а"), _T("ов")
};

class CMyListCtrl;

struct DiskCatalog
{
	vector<FileRecord> vecFC; //каталог
	UINT nVectorPos; //глобальная позиция для поиска в каталоге

	//переменные для отображения информации о ФС диска
	int nFreeRecs; //количество свободных записей в каталоге
	int nFreeBlocks; //количество свободных блоков/кластеров (зависит от ФС) на основе чего высчитывается кол-во свободных байтов
};

class CBKDImage
{
protected:
	CMyListCtrl* m_ParentLisctCtrl;

	BOOL m_bFileOpened; //флаг, означает что в g_ImgFile открыт файл, т.к. чего-то я не нашёл стандартных средств у CFile,
						//чтобы узнать открыт там файл или надо сперва сделать close, чтобы потом сделать open.
	BOOL m_bFileROMode;
	CString m_strImgPathName; //имя файла образа
	CString m_strStorePath; //путь, куда будем сохранять файлы

	WORD m_mBinHeader[2];
	BYTE m_mSector[512];
	BYTE m_mBlock[COPY_BLOCK_SIZE];

	DiskCatalog		m_sDiskCat; //структура, куда собрано всё, что касается каталога
	IMAGE_TYPES		m_ImgType;
	long			m_nBaseOffset;
	ULONGLONG		m_nSize;	//сохраняем размер образа для правильной обработки логических дисков.
	//специально для вьювера, нужны переменные
	LONGLONG m_tPos; //начальная позиция в файле
	int m_tLen; //начальная длина в файле

public:
	CBKDImage(CMyListCtrl* pLC, long nBaseOffset, CString strFileName);
	virtual ~CBKDImage(void);
	BOOL OpenFile(void);
	void CloseFile(void);
	void Reset(void);
	inline long GetBaseOffset(void){ return m_nBaseOffset; }
	inline const CString& GetCurrImgName(void) { return m_strImgPathName; }
	inline IMAGE_TYPES GetImgType(void){ return m_ImgType;}
	inline void SetImgType(IMAGE_TYPES type){ m_ImgType = type;}
	void SetStorePath(const CString& str);
	void SetSafeName(CString& str);
	void SetSafeDir(CString& str);
	inline void SetImgSize(ULONGLONG size) { m_nSize = size; }
	inline ULONGLONG GetImgSize(void) { return m_nSize; }
	//прочитаем весь каталог в массив
	virtual BOOL ReadCatalog();
	//получим номер окончания слова в зависимости от числа num
	int GetWordEndIdx(int num);
	//выдадим информацию о диске
	virtual CString GetInfo();
	//извлечение файла из образа
	virtual void ExtractObject(FileRecord *sFile, BOOL bExtractAsBin = FALSE);
	virtual void ExtractFile(FileRecord *sFile, BOOL bExtractAsBin = FALSE);
	//просмотр файла из образа
	virtual void ViewFile(FileRecord *sFile);
	//virtual void ViewFileAsSprite(FileRecord *sFile);
	//добавление файла в образ
	virtual FILESAVE_STATUS AddFile(CString& str, int nDirNum, BOOL bAddAsBin = FALSE);
	//получение данных специфических для заданной ОС
	virtual CString GetSpecificData(DWORD dd){ return _T("");}

	/*
	получение параметров файла или директории из текущей директории
	выход - FALSE - конец, имена кончились
			TRUE - ещё не конец, есть ещё имена в директории
	*/
	BOOL GetStartFileName(FileRecord *sFile, int iDirNum);
	BOOL GetNextFileName(FileRecord *sFile, int iDirNum);

protected:

	CString BKToUNICODE (BYTE* pBuff, int size);
	TCHAR BKToUNICODE_Byte (BYTE b);

	CString BKToUNICODE11 (BYTE* pBuff, int size);
	TCHAR BKToUNICODE_Byte11 (BYTE b);

	void UNICODEtoBK(CString& ustr, BYTE* pBuff, int bufSize, BOOL bFillBuf = FALSE);

	virtual void SaveCurrPos(CFile *f, int len);
	virtual int SeekToPos(CFile *f, int pos);
	virtual void SeekToBeginFile(CFile *f, FileRecord *fr);
	virtual BYTE GetByte(CFile *f, int *len);
	virtual WORD GetWord(CFile *f, int *len);
	virtual void PutTchar(CFile *f, TCHAR tch, int *counter);
};

