#include "StdAfx.h"
#include "BKDMkdos.h"


CBKDMkdos::CBKDMkdos(CMyListCtrl* pLC, long nBaseOffset, CString strFileName): CBKDImage(pLC, nBaseOffset, strFileName)
{
}


CBKDMkdos::~CBKDMkdos(void)
{
}

BOOL CBKDMkdos::ReadCatalog()
{
	CBKDImage::ReadCatalog();

	if(!m_bFileOpened)
	{
		return FALSE;
	}
	MKDosFileRecord curr_record;
	FileRecord curr_fr;

	//  
	g_ImgFile.Seek(m_nBaseOffset + 0, CFile::begin);
	if (g_ImgFile.Read (m_mSector, sizeof(m_mSector)) < sizeof(m_mSector))
	{
		return FALSE;
	}
	int files_count = 0, recs_count = 0;
	int files_total = *(WORD*)&m_mSector[FMT_MKDOS_CAT_RECORD_NUMBER]; //  - . ( !)
	int data_begin = *(WORD*)&m_mSector[FMT_MKDOS_FIRST_FILE_BLOCK] * 512; //  
	m_sDiskCat.nFreeRecs = (data_begin - FMT_MKDOS_CAT_BEGIN) / sizeof(MKDosFileRecord); //    ,        
	int used_size = 0;

	g_ImgFile.Seek(m_nBaseOffset + FMT_MKDOS_CAT_BEGIN, CFile::begin); //   .

	while (files_count <= files_total)
	{
		if (g_ImgFile.Read (&curr_record, sizeof(MKDosFileRecord)) < sizeof(MKDosFileRecord))
		{
			return FALSE; //    ,    ,   ,
			//    .
		}
		//  ,       
		if (curr_record.status == 0 && curr_record.name[0] == 0)
		{
			break; //   
		}

		recs_count++;

		curr_fr.nAttr = 0;
		if (curr_record.status == 0377)
		{
			curr_fr.nAttr |= FR_ATTR_DELETED;
			recs_count--; // 
		}
		else if (curr_record.status == 0200)
		{
			curr_fr.nAttr |= FR_ATTR_BAD;
			if (curr_record.name[0] == 0)
			{
				curr_record.name[0] = 'B';
				curr_record.name[1] = 'A';
				curr_record.name[2] = 'D';
			}
			//    ,     
		}
		else
		{
			files_count++;
		}

		if (curr_record.name[0] == 0177)
		{ // 
			curr_fr.nAttr |= FR_ATTR_DIR;
			curr_fr.nRecType = RECORD_TYPE_DIR;
			curr_fr.strName = BKToUNICODE((BYTE*)curr_record.name + 1, 13).Trim();
			curr_fr.iDirNum = curr_record.dir_num;
			curr_fr.iDirDir = curr_record.status;
			curr_fr.iBlkSize = 0;
		}
		else
		{ // 
			CString name = BKToUNICODE((BYTE*)curr_record.name, 14).Trim();
			CString ext = _T("");
			int l = name.GetLength();
			int t = name.ReverseFind(_T('.'));
			if (t >= 0) //  
			{
				ext = name.Right(l - t - 1);
				name = name.Left(t).Trim();
			}
			if (!ext.IsEmpty())
			{
				name += _T(".") + ext;
			}
			curr_fr.strName = name;
			curr_fr.iDirNum = curr_record.dir_num;
			curr_fr.iDirDir = 0;
			if (curr_fr.nAttr & FR_ATTR_DELETED)
			{
				curr_fr.iDirNum = 0; //      
				curr_fr.nRecType = RECORD_TYPE_FILE;
				curr_fr.iBlkSize = curr_record.len_blk;
			}
			else
			{
				if (curr_record.status == 2)
				{ //     .
					//  ,  .       .
					curr_fr.nAttr |= FR_ATTR_LOGDISK;
					curr_fr.nRecType = RECORD_TYPE_LOGDSK;
				}
				else
				{
					curr_fr.nRecType = RECORD_TYPE_FILE;
				}
				if (curr_record.status == 1)
				{
					curr_fr.nAttr |= FR_ATTR_PROTECTED;
				}
				used_size += curr_record.len_blk;
				curr_fr.iBlkSize = curr_record.len_blk;
			}
			
		}

		curr_fr.iAddress = curr_record.address;

		if (curr_fr.nAttr & FR_ATTR_LOGDISK) //    ,      .
		{
			curr_fr.iSize = curr_record.len_blk * 512;
		}
		else
		{
			/*	   .
				..  curr_record.length      0200000,       
				   0200000 ..      128 .(128 == 65536.== 0200000) 
		
			hw = (curr_record.len_blk >>7 ) << 16; //    
			hw = (curr_record.len_blk / 128) * 65536 = curr_record.len_blk * 512
			*/
			DWORD hw = (curr_record.len_blk  << 9) & 0xffff0000; //    
			curr_fr.iSize = hw + curr_record.length; // . + .
		}
		curr_fr.iStartSector = curr_record.start_blok;
		m_sDiskCat.vecFC.push_back(curr_fr);

	}
	m_sDiskCat.nFreeRecs -= recs_count;
	m_sDiskCat.nFreeBlocks = *(WORD*)&m_mSector[FMT_MKDOS_DISK_SIZE] - *(WORD*)&m_mSector[FMT_MKDOS_FIRST_FILE_BLOCK] - used_size;
	return TRUE;
}

FILESAVE_STATUS CBKDMkdos::AddFile(CString& str, int nDirNum, BOOL bAddAsBin)
{
	if (m_bFileROMode)
	{ //     ,
		return FS_IMAGE_RO; //       .
	}
	CFile inFile;
	FILESAVE_STATUS bRet = FS_IO_ERROR;
	int FileAddr, FileSize;

	UINT CatOffs = 0; //   ,     .
	UINT nCatSize = 20*512;	//     . //*(WORD*)&m_mSector[FMT_MKDOS_FIRST_FILE_BLOCK] * 512;
							//   ,     
	MKDosFileRecord mFR;
	ZeroMemory(&mFR, sizeof(MKDosFileRecord));

	if (inFile.Open (str, CFile::modeRead | CFile::shareDenyWrite))
	{
		FileSize = (int)inFile.GetLength();
		CString name = inFile.GetFileName();
		CString ext = _T("");
		int l = name.GetLength();
		int t = name.ReverseFind(_T('.'));
		if (t >= 0) //  
		{
			ext = name.Right(l - t - 1);
			name = name.Left(t);
		}
		//,      ?
		inFile.Read(m_mBinHeader, sizeof(m_mBinHeader));
		if (((ext.CompareNoCase(_T("BIN"))== 0) || bAddAsBin) && (m_mBinHeader[1] == FileSize - 4)) //    
		{ //,  
			FileAddr = m_mBinHeader[0];
			FileSize = m_mBinHeader[1];
			//      .bin
			t = name.ReverseFind(_T('.'));
			if (t >= 0)
			{
				l = name.GetLength();
				ext = name.Right(l - t - 1);
				name = name.Left(t);
			}
		}
		else
		{ //,  
			inFile.Seek(0, CFile::begin);
			FileAddr = 01000;
		}
		if (!ext.IsEmpty())
		{
			name += _T(".") + ext;
		}
		UNICODEtoBK(name, mFR.name, 14, TRUE);
		mFR.dir_num = nDirNum; //   
		mFR.address = FileAddr; //,     ,    
		mFR.len_blk = FileSize / 01000; //  
		mFR.length = FileSize % 0200000; //     65535
		if (m_sDiskCat.nFreeBlocks * 01000 < ((FileSize) ? (((FileSize - 1) | 0777) + 1) : 0))
		{
			inFile.Close();
			return FS_DISK_FULL;
		}

	}
	else
		return FS_FILE_OPENERR;

	BYTE* pCat = new BYTE[nCatSize];
	ZeroMemory(pCat, nCatSize);
	MKDosFileRecord *pMKCat = (MKDosFileRecord *)(pCat + FMT_MKDOS_CAT_BEGIN);
	UINT nMKCatSize = (nCatSize - FMT_MKDOS_CAT_BEGIN) / sizeof(MKDosFileRecord);
	/*
	1.  .
	2.     
	3.  , ,      
	4.   ,  
	5.    ,   
	6.  .
	7.  .
	*/
	BOOL bMode = FALSE; // ,     , ..    
	//   
	g_ImgFile.Seek(m_nBaseOffset + 0, CFile::begin);
	// 
	if (nCatSize == g_ImgFile.Read (pCat, nCatSize))
	{
		//,      .
		UINT tc = 0;
		BOOL bFound = FALSE;
		while (tc < nMKCatSize)
		{
			if ((pMKCat[tc].status == 0) && ((pMKCat[tc].name[0] | pMKCat[tc].name[1]) == 0))
			{
				break; //   ,   
			}
			if (pMKCat[tc].status == 0377)
			{
				//   - ,  
			}
			else if (!strncmp((char*)mFR.name, (char*)pMKCat[tc].name, 14))	//  
			{
				bFound = TRUE;
				break;
			}
			tc++;
		}
		if (bFound)
		{
			bRet = FS_FILE_EXIST;
		}
		else
		{
			//    .
			bFound = FALSE;
			tc = 0;
			UINT nFirstFreeBlock = 0;
			while (tc < nMKCatSize)
			{
				if ((pMKCat[tc].status == 0377) && (pMKCat[tc].len_blk >= mFR.len_blk))
				{ //    
					bFound = TRUE;
					bMode = FALSE;
					CatOffs = tc;
					nFirstFreeBlock = pMKCat[tc].start_blok;
					break;
				}
				if ((pMKCat[tc].status == 0) && ((pMKCat[tc].name[0] | pMKCat[tc].name[1]) == 0))
				{ //   
					bFound = TRUE;
					bMode = TRUE;
					CatOffs = tc;
					nFirstFreeBlock = pMKCat[tc].start_blok;
					break;
				}
				tc++;
			}
			if (bFound)
			{
				// 
				mFR.start_blok = nFirstFreeBlock;
				//  
				g_ImgFile.Seek(m_nBaseOffset + nFirstFreeBlock * 512, CFile::begin);
				int nReaded;
				do {
					ZeroMemory(m_mBlock, COPY_BLOCK_SIZE);
					nReaded = inFile.Read(m_mBlock, COPY_BLOCK_SIZE);
					//    .
					g_ImgFile.Write(m_mBlock, ((nReaded) ? (((nReaded - 1) | 0777) + 1) : 0));
					FileSize -= nReaded;
				} while(FileSize > 0);
				bRet = FS_NO_ERROR;
			}
			else
				bRet = FS_CAT_FULL;
		}
	}
	inFile.Close();
	if (bRet == FS_NO_ERROR)
	{ //   ,  
		*(WORD*)&pCat[FMT_MKDOS_CAT_RECORD_NUMBER] += 1;
		*(WORD*)&pCat[FMT_MKDOS_TOTAL_FILES_USED_BLOCKS] += mFR.len_blk;
		UINT nHoleZize;
		if (bMode)
		{	//      
			nHoleZize = pMKCat[CatOffs].len_blk;
			if (CatOffs + 1 > nMKCatSize) //     
			{
				bRet = FS_CAT_FULL;
			}
			else
			{
				//  
				pMKCat[CatOffs] =  mFR;
				//   
				MKDosFileRecord hole;
				ZeroMemory(&hole, sizeof(MKDosFileRecord));
				hole.start_blok = mFR.start_blok + mFR.len_blk;
				hole.len_blk = nHoleZize - mFR.len_blk;
				//      
				pMKCat[CatOffs + 1] = hole;
			}
			//,       .     , 
			//    .
		}
		else
		{	//    ,   ,   ,    
			if ((nHoleZize = pMKCat[CatOffs].len_blk) > mFR.len_blk)
			{ // 
				UINT tc = CatOffs + 1;
				//  
				while (tc < nMKCatSize)
				{
					if ((pMKCat[tc].status == 0) && ((pMKCat[tc].name[0] | pMKCat[tc].name[1]) == 0))
						break;
					tc++;
				}
				tc++;
				if (tc >= nMKCatSize) //   
				{
					bRet = FS_CAT_FULL;
				}
				//          , ..    .
				else
				{	//
					while (tc > CatOffs)
					{
						pMKCat[tc] = pMKCat[tc - 1];
						tc--;
					}
					//  
					pMKCat[CatOffs] = mFR;
					MKDosFileRecord hole;
					ZeroMemory(&hole, sizeof(MKDosFileRecord));
					hole.status = 0377;
					hole.dir_num = 0377;
					hole.start_blok = mFR.start_blok + mFR.len_blk;
					hole.len_blk = nHoleZize - mFR.len_blk;
					strcpy_s((char*)hole.name, 14, "<HOLE>");
					hole.length = (hole.len_blk * 512) % 0200000;
					//     
					pMKCat[CatOffs + 1] = hole;
				}
			}
			else
			{	//   , 
				//  
				pMKCat[CatOffs] = mFR;
			}
		}
		if (bRet == FS_NO_ERROR)
		{
			//  
			g_ImgFile.Seek(m_nBaseOffset + 0, CFile::begin);
			g_ImgFile.Write(pCat, nCatSize);
		}
	}
	delete [] pCat;
	return bRet;
}
/*
      ,     
      .
*/

