#include "StdAfx.h"
#include "BKDNCdos.h"


CBKDNCdos::CBKDNCdos(CMyListCtrl* pLC, long nBaseOffset, CString strFileName): CBKDImage(pLC, nBaseOffset, strFileName)
{
	m_pDiskMap = new BYTE[DISKMAP_SIZE];
	ZeroMemory(m_pDiskMap,DISKMAP_SIZE);
}


CBKDNCdos::~CBKDNCdos(void)
{
	if (m_pDiskMap)
		delete m_pDiskMap;
}


BOOL CBKDNCdos::ReadCatalog()
{
	CBKDImage::ReadCatalog();

	if(!m_bFileOpened)
	{
		return FALSE;
	}
	NCdosFileRecord curr_record;
	FileRecord curr_fr;

	//     .
	//  1 
	if (m_nBaseOffset + 512 != g_ImgFile.Seek(m_nBaseOffset + 512, CFile::begin))
	{
		return FALSE;
	}
	if (g_ImgFile.Read (m_pDiskMap, DISKMAP_SIZE) < DISKMAP_SIZE)
	{
		return FALSE;
	}
	//    ,   .
	//  ,    
	if (g_ImgFile.Read (&curr_record, sizeof(NCdosFileRecord)) < sizeof(NCdosFileRecord))
	{
		return FALSE; //    ,    ,   ,
		//    .
	}
	int used_size = 0;
	int files_total = 2;
	while (1)
	{
		if (g_ImgFile.Read (&curr_record, sizeof(NCdosFileRecord)) < sizeof(NCdosFileRecord))
		{
			return FALSE; //    ,    ,   ,
			//    .
		}
		if (curr_record.name[0] == 0)
		{
			break; //    
		}

		if (curr_record.name[12] == '.')
		{
			curr_fr.strName = BKToUNICODE((BYTE*)curr_record.name, 12).Trim();
			curr_fr.strName += BKToUNICODE((BYTE*)curr_record.name + 12, 4).Trim();
		}
		else
		{
			curr_fr.strName = BKToUNICODE((BYTE*)curr_record.name, 16).Trim();
		}

		curr_fr.nAttr = 0;
		curr_fr.nRecType = RECORD_TYPE_FILE;
		curr_fr.iDirNum = 0;
		curr_fr.iDirDir = 0;
		curr_fr.iAddress = curr_record.address;
		curr_fr.iSize = curr_record.length;
		curr_fr.iBlkSize = (curr_fr.iSize) ? ((((curr_fr.iSize - 1) | 0777) + 1) / 01000) : 0;
		used_size += curr_fr.iBlkSize;

		//   .    
		curr_fr.iStartSector = files_total;
		int n = GetStartBlock(files_total);
		if (m_pDiskMap[n] & 0x80)
		{
			curr_fr.nAttr |= FR_ATTR_PROTECTED;
		}

		m_sDiskCat.vecFC.push_back(curr_fr);
		files_total++;
	}
	m_sDiskCat.nFreeRecs = ((010000 - 02440) / sizeof(NCdosFileRecord)) - files_total - 1;
	m_sDiskCat.nFreeBlocks = 1600 - 8 - used_size; //TODO  -   ?
	return TRUE;
}

void CBKDNCdos::ExtractFile(FileRecord *sFile, BOOL bExtractAsBin)
{
	CFile outFile;

	CString fname = sFile->strName;
	SetSafeName(fname);

	if (outFile.Open (m_strStorePath + fname + ((bExtractAsBin) ? _T(".bin") : _T("")), CFile::modeCreate|CFile::modeWrite))
	{
		m_currBlock = GetStartBlock(sFile->iStartSector);
		int len = sFile->iSize;
		int out_len;
		if (bExtractAsBin)
		{
			m_mBinHeader[0] = (WORD)sFile->iAddress;
			m_mBinHeader[1] = (WORD)sFile->iSize; //  ,    64,  .
			outFile.Write(m_mBinHeader, sizeof(m_mBinHeader));
		}
		while((len > 0) && (m_currBlock != 0))
		{
			SeekToBlock(&g_ImgFile);
			g_ImgFile.Read (m_mBlock, BLOCK_SIZE);

			if (len >= BLOCK_SIZE)
			{
				out_len = BLOCK_SIZE;
			}
			else
			{
				out_len = len;
			}
			outFile.Write(m_mBlock, out_len);
			len -= out_len;
			m_currBlock = GetNextBlock(m_currBlock);
		}

		outFile.Close();
	}
}

void CBKDNCdos::SeekToBlock(CFile *f)
{
	m_nLenBlock = BLOCK_SIZE;
	if (m_currBlock)
	{
		//int sector = m_currBlock * 2;
		m_tPos = f->Seek(m_nBaseOffset + m_currBlock * 1024, CFile::begin);
	}
}

int CBKDNCdos::GetStartBlock(int fn)
{
	int n = 0; //   
	BOOL bfind = FALSE;
	while (n < DISKMAP_SIZE)
	{
		if ((m_pDiskMap[n] & 0x7f) == fn)
		{
			bfind = TRUE;
			break;
		}
		n++;
	}
	if (bfind)
	{
		return n;
	}
	return 0;
}

int CBKDNCdos::GetNextBlock(int blk)
{
	int fn = m_pDiskMap[blk] & 0x7f;
	int n = blk + 1; //   
	BOOL bfind = FALSE;
	while (n < DISKMAP_SIZE)
	{
		if ((m_pDiskMap[n] & 0x7f) == fn)
		{
			bfind = TRUE;
			break;
		}
		n++;
	}
	if (bfind)
	{
		return n;
	}
	return 0;
}

int CBKDNCdos::SeekToPos(CFile *f, int pos)
{
	//     ,    
	while (pos > BLOCK_SIZE)
	{
		pos -= BLOCK_SIZE;
		m_currBlock = GetNextBlock(m_currBlock);
		m_tLen -= BLOCK_SIZE;
	}
	SeekToBlock(f);
	//    
	g_ImgFile.Seek( m_tPos + pos, CFile::begin);
	m_nLenBlock -= pos;
	return m_tLen - pos;
}

void CBKDNCdos::SeekToBeginFile(CFile *f, FileRecord *fr)
{
	m_currBlock = GetStartBlock(fr->iStartSector);
	SeekToBlock(f);
}

BYTE CBKDNCdos::GetByte(CFile *f, int *len)
{
	BYTE b;
	f->Read (&b, sizeof(b));
	*len -= sizeof(b);
	m_nLenBlock -= sizeof(b); //    
	if (m_nLenBlock <= 0)
	{ //    
		m_currBlock = GetNextBlock(m_currBlock);
		SeekToBlock(f);
	}
	return b;
}

WORD CBKDNCdos::GetWord(CFile *f, int *len)
{
	WORD w;
	f->Read (&w, sizeof(w));
	*len -= sizeof(w);
	m_nLenBlock -= sizeof(w); //    
	if (m_nLenBlock <= 0)
	{ //    
		m_currBlock = GetNextBlock(m_currBlock);
		SeekToBlock(f);
	}
	return w;
}



