#include "StdAfx.h"
#include "BKDRT11.h"


CBKDRT11::CBKDRT11(CMyListCtrl* pLC, long nBaseOffset, CString strFileName): CBKDImage(pLC, nBaseOffset, strFileName)
{
}


CBKDRT11::~CBKDRT11(void)
{
}

BOOL CBKDRT11::ReadCatalog()
{
	CBKDImage::ReadCatalog();

	if(!m_bFileOpened)
	{
		return FALSE;
	}

	RT11FileRecord curr_record;
	RT11SegmentHdr curr_segment_header;
	FileRecord curr_fr;

	int segment = 0; //  ,    1, .. 0 -    
	int begin_block = 0; //,    
	CString ext;

	//  1 
	if (m_nBaseOffset + 512 != g_ImgFile.Seek(m_nBaseOffset + 512, CFile::begin))
	{
		return FALSE;
	}
	//  ,      2 
	if (g_ImgFile.Read (m_mSector, sizeof(m_mSector)) < sizeof(m_mSector))
	{
		return FALSE;
	}
	if (m_mSector[0726] == 0x8e53)
	{ //  
		segment = m_mSector[0722]; //      
		begin_block = m_mSector[0724];
	}
	else
	{
		segment = 1; //,  ,       .
		begin_block = 6;
	}

	int recs_count = 0; //- 
	int used_size = 0;
	//    
	while (segment > 0)
	{ //  .     .
		//   
		int offs = ((segment - 1) * 2 + begin_block) * 512;
		if (m_nBaseOffset + offs != g_ImgFile.Seek(m_nBaseOffset + offs, CFile::begin))
		{
			return FALSE;
		}
		//  ,      ,    
		// -  .
		if (g_ImgFile.Read (&curr_segment_header, sizeof(RT11SegmentHdr)) < sizeof(RT11SegmentHdr))
		{
			return FALSE;
		}

		//    .
		int file_start_block = curr_segment_header.file_block;

		while (1)
		{
			if (g_ImgFile.Read (&curr_record, sizeof(RT11FileRecord)) < sizeof(RT11FileRecord))
			{
				return FALSE;
			}

			//  .   ,   . ..     ,   
			if (curr_segment_header.filerec_add)
			{
				g_ImgFile.Seek(curr_segment_header.filerec_add * 2, CFile::current);
			}
			if (curr_record.status & RT11_FILESTATUS_EOS)
			{
				break; //  .     
			}
			recs_count++;
			curr_fr.strName = DecodeRadix50(curr_record.name, 2).Trim();
			ext = DecodeRadix50(&curr_record.ext, 1).Trim();
			if (!ext.IsEmpty())
			{
				curr_fr.strName += _T(".") + ext;
			}
			//  .
			curr_fr.nAttr = 0;
			if (curr_record.status & RT11_FILESTATUS_TEMPORARY)
			{
				curr_fr.nAttr |= FR_ATTR_TEMPORARY;
			}
			if (curr_record.status & RT11_FILESTATUS_PROTECTED)
			{
				curr_fr.nAttr |= FR_ATTR_PROTECTED;
			}
			if (curr_record.status & RT11_FILESTATUS_UNUSED)
			{
				curr_fr.nAttr |= FR_ATTR_DELETED;
				recs_count--;
			}
			if (ext == _T("DSK"))
			{
				curr_fr.nAttr |= FR_ATTR_LOGDISK;
				curr_fr.nRecType = RECORD_TYPE_LOGDSK;
			}
			else
			{
				curr_fr.nRecType = RECORD_TYPE_FILE;
			}
			curr_fr.iDirNum = 0; // 
			curr_fr.iDirDir = 0;
			curr_fr.iAddress = 0; // 11     , .. ,      .
			curr_fr.iSize = curr_record.blk_size * 512;
			//curr_fr.nSpecificData = 0;
			curr_fr.nSpecificData = (curr_record.task_num << 24) | (curr_record.chan_num << 16) | curr_record.date;
			curr_fr.iStartSector = file_start_block;
			if (!(curr_fr.nAttr & FR_ATTR_DELETED)) used_size += curr_record.blk_size;

			file_start_block += curr_record.blk_size;
			curr_fr.iBlkSize = curr_record.blk_size;

			m_sDiskCat.vecFC.push_back(curr_fr);
		}

		segment = curr_segment_header.next_segment;
	}
	m_sDiskCat.nFreeRecs = recs_count;
	m_sDiskCat.nFreeBlocks = used_size;
	return TRUE;
}


CString CBKDRT11::GetInfo()
{
	CString str, strf;
	//strf.LoadString(IDS_INFO_USED_RT11);
	int bytes = m_sDiskCat.nFreeBlocks * 512;
	str.Format(strf, m_sDiskCat.nFreeRecs, tblStrRec[GetWordEndIdx(m_sDiskCat.nFreeRecs)], m_sDiskCat.nFreeBlocks, tblStrBlk[GetWordEndIdx(m_sDiskCat.nFreeBlocks)], bytes, tblStrBlk[GetWordEndIdx(bytes)]);
	return str;
}


/*
   radix50.
: buf -  ,
	len -    
*/
CString CBKDRT11::DecodeRadix50(WORD *buf, int len)
{
	CString ret = _T("");
	for (int i = 0; i < len; i++)
	{
		WORD w = buf[i];
		//   ,     
		TCHAR tch3 = RADIX50[w % 050];
		w /= 050;
		TCHAR tch2 = RADIX50[w % 050];
		w /= 050;
		TCHAR tch1 = RADIX50[w % 050];

		ret += tch1;
		ret += tch2;
		ret += tch3;
	}
	return ret;
}

CString CBKDRT11::GetSpecificData(DWORD dd)
{
	CString str;
	int nTask = (dd >> 24) & 0xff;
	int nChan = (dd >> 16) & 0xff;
	int nDay = (dd >> 5) & 0x1f;
	int nMon = (dd >> 10) & 0xf;
	int nYear = (dd & 0x1f) + 1972;
	str.Format(_T("T%d:C%d; %02d.%02d.%4d"), nTask, nChan, nDay, nMon, nYear);
	return str;
}


